/********************************************************************
	INTRODUCTION TO MEMORY MANAGEMENT ON PLUTO6

1) SRAM memory 256k
The range of memory for SRAM battery backed ram goes from
0x10000000 to 0x1003ffff and in this range of memory three
sections are created
	a) KEEPRAM for bootloader:
	The section is inderictly visible from there. As section 
	starting at 0x10000000 of size 0x400 (1k) was created from
	the bootloader. this section was used to store the backed up
	data from the idram.c and sysram.c of the bootloader
	b) KEEPRAM for game:
	create a section from 0x10000400 of size 0x10000 (65k).
	this section is used to store the backed up data from the
	idram.c and sysram.c used in the game code
	c) RAM for game and bootloader:
	create a section from 0x10010400 of size 0x2fc00 (190k)
	this section is used to store all other data from game code
2) Stack
The coldfire holds a 8k byte module of SRAM. As this module is
directly connected to the processor, access to data is the fastest.
For this reason, heber is using this area (stacka) to store the
stack.
Rather that using the whole 8k, 256 bytes are left free (fram) in
case some of this memory needs to be used in the future.
3) DRAM (2 meg)
Heber has allocated a single segment into the dram to store the game
code. As it is an area of RAM, it could be used for other means such
as stroring data that are not fitting into the SRAM. Nevertheless,
it is recommended to contact Technical Support to help through the
process as modifying this linker could stop the board from working
correctly
********************************************************************/

OUTPUT_FORMAT("coff-m68k")
MEMORY 
{
        rom	: ORIGIN = 0x60000000, LENGTH = 0x001fffff
        keepram	: ORIGIN = 0x10000400, LENGTH = 0x00010000
        ram	: ORIGIN = 0x10010400, LENGTH = 0x0002fc00
        stacka  : ORIGIN = 0x50000000, LENGTH = 0x00001f00
	fram	: ORIGIN = 0x50001f00, LENGTH = 0x000000ff
}
SECTIONS
{

/********************************************************************* 
1) fill the dram memory ram with:
	a) the vector table from clodfire
	b) library and user code. Code extends
	to constant variables.
*********************************************************************/
	.vectors :
	{
		vectors_start	= .;
                ../../libs/hardware/pluto6/81-16616-25/vectors.o(.text)
		vectors_end		= .;
	} > rom

	.text :
	{
		text_start		= .;
		checksum_start	= .;
		*(.text)

		__CTOR_LIST__ = .;
		LONG((__CTOR_END__ - __CTOR_LIST__) / 4 - 2)


		*(.ctors)
		LONG(0)
		__CTOR_END__ = .;

		__DTOR_LIST__ = .;
		LONG((__DTOR_END__ - __DTOR_LIST__) / 4 - 2)
		*(.dtors)
		LONG(0)
		__DTOR_END__ = .;

		*(.lit)
		*(.shdata)
		.endcode		= .;
		endcode 		= .;
	  }  > rom

	.shbss :
	{
		*(.shbss)
		text_end		= .;
		romdata_start	= .;
	} > rom


/******************************************************************** 
2) fill the backup memory (keepram) with uninitialised variables 
from files idram.c and sysram.c
*********************************************************************/
	/* all unitialised data from idram.c file. */
	.id :
	{
		id_start	= .;
		*(.id)
		/***********************
		filename(COMMON) refers to uninitialized
		data from a particular file
		***********************/
		idram.o(COMMON);
		id_end		= .;
	} > keepram

	/* all unitialised data from idram.c file. */
	.system :
	{
		system_start	= .;
		*(.system)
		/***********************
		filename(COMMON) refers to uninitialized
		data from a particular file
		***********************/
		sysram.o(COMMON);
		system_end		= .;
	} > keepram

/********************************************************************* 
3) fill the sram with variables
a) the data segment which is the collection of all 
   initialised global variables (ex: ULONG test_variable = 50;)
b) the bss segment is for static uninitialized data 
   (ex: static ULONG test_variable;)
   Be carrefull, if this data is intitialised (ex: static ULONG
   test_variable = 50;), it will be moved into the data section but
   it will not appear visible
c) the common segment which is the collection of all
   uninitialised global variables (ex: ULONG test_variable;)

*********************************************************************/
	.data : AT (ADDR(.shbss) + SIZEOF(.shbss))
	{
		ram_start	= .;
		vars_start	 	= .;
		*(.data)
		vars_end		=  .;
	} > ram

	checksum_end		= romdata_start + SIZEOF(.data);

	.bss ram_start + 0x00004000 :
	{
		__bss_start = .;
		bss_start	= .;
		zvars_start	 	= .;
		*(.bss)
		bss_end		= .;
		end = ALIGN(0x4);
		_end = ALIGN(0x4);
		endbssram = . ;
	} > ram

	COMMON :
	{
		__common_start	= .;
		*(COMMON)
		end = ALIGN(0x4);
		_end = ALIGN(0x4);
		zvars_end	 	= .;
		zvars_size		= zvars_end - zvars_start;
		endram = . ;
	} > ram

/********************************************************************* 
 	Allocation of the heap and stack 
*********************************************************************/
	.heap :
	{
		heap_start	= .;
		. = . + 0x00005fff;
		heap_end	= .;
	} > ram

	.stack :
	{
		stack_start	= .;
		. = . + 0x00001e00;
		stack_end	= .;
	} > stacka

/********************************************************************* 
 	section used by heber and compiler
*********************************************************************/
	.safe :
	{
		safe_start	= . ;
		. = . + 0x000000ff;
		safe_end	= . + 1;
	} > ram

	.stab  . (NOLOAD) :
	{
		[ .stab ]
	}
	
	.stabstr  . (NOLOAD) :
	{
		[ .stabstr ]
	}
}
